﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("FuctionalDistance", "A distance wrapper for DataPoints")]
  internal class FuctionalDistance<T> : DistanceBase<T> where T : class {

    #region properties
    [Storable]
    private Func<T, T, double> dist;
    #endregion

    #region HLConstructors & Cloning
    [StorableConstructor]
    protected FuctionalDistance(bool deserializing) : base(deserializing) { }
    protected FuctionalDistance(FuctionalDistance<T> original, Cloner cloner) : base(original, cloner) {
      dist = original.dist; //func needs to be stateless
    }
    public override IDeepCloneable Clone(Cloner cloner) { return new FuctionalDistance<T>(this, cloner); }
    public FuctionalDistance(Func<T, T, double> distance) {
      dist = distance;
    }
    #endregion

    public override double Get(T a, T b) {
      return dist.Invoke(a, b);
    }

    public override double Get(T a, T b, double threshold) {
      return Get(a, b);   //no threshold evaluation here
    }
  }
}

